
local MAJOR = "StarLibError-1.0" 
local MINOR = 1
assert(LibStub, MAJOR.." requires LibStub") 
local StarLibError = LibStub:NewLibrary(MAJOR, MINOR)
if not StarLibError then return end
local pool = setmetatable({}, {__mode = "k"})
StarLibError.defaultTexts = {"Quiet",  "Normal", "Verbose"}

local DEFAULT_LEVEL = #StarLibError.defaultTexts -- verbose
-- 2 for warning
-- 1 for quiet

if not StarLibError.__index then
	StarLibError.__index = StarLibError
	StarLibError.name = MAJOR
	StarLibError.errorLevel = DEFAULT_LEVEL
end

-- Create a new StarLibError object
-- @name StarLibError:New
-- @usage StarLibError:New(name, [handler])
-- @param name Give your error object a name
-- @param errorLevel Critical level for this object. This is optional. Default is 2.
-- @param handler An optional handler function, otherwise we use ChatFrame1 for printing
-- @return A new StarLibError object
function StarLibError:New(name, errorLevel, handler) 
	if not name then
		self:Print("StarLibError:New(name, level, handler) received name == nil")
		return
	end
	
	local obj = next(pool)

	if obj then
		pool[obj] = nil
	else
		obj = {}
	end

	setmetatable(obj, self)
	
	obj.name = name
	obj.errorLevel = errorLevel or DEFAULT_LEVEL
	
	if not handler then
		obj.handler = self.Print
	else
		obj.handler = handler	
	end
	
	obj.__call = function(...)
		obj:handler(...)
	end
	
	return obj	
end

-- Delete a StarLibError object
-- @name StarLibError:Del
-- @usage object:Del() or StarLibError(object)
-- @return Nothing
function StarLibError:Del(ev)
	pool[ev] = true
end

-- Print something
-- @name StarLibError.Print
-- @usage LibError:Print(text, errorLevel)
-- @param text The text to print.
-- @param errorLevel The critical threshhold for this LibError object
-- @return Nothing
function StarLibError:Print(text, errorLevel)
	assert(self, "StarLibError failed printing due to invalid 'self'")
	if not errorLevel then
		errorLevel = self.errorLevel
	end
	local color
	if errorLevel == 3 then
		color = "|cffff0000"
	elseif errorLevel == 2 then
		color = "|cffffff00"
	else
		return
	end
	if errorLevel > self.errorLevel then
		return
	end
	if not self.name or self.name == "" then 
		ChatFrame1:AddMessage("StarLibError object has no name. Type of 'self' " .. type(self))
		assert(self.name, "Check ChatFrame1")
		return
	end
	assert(text, "StarLibError failed printing due to nil text (" .. self.name .. ")")
	if type(self.handler) == "function" and not self.hasExternalHandler then
		ChatFrame1:AddMessage(color .. (self.name or " ") .. "|r:: " .. text)
	elseif type(self.handler) == "function" then
		self.handler(self.data)
	end
end

StarLibError.handler = StarLibError.Print